// page 272-283

import { Injectable } from "@angular/core";
import { Observable, Subject } from "rxjs";
import { Message } from "./message.model";

// not on page 282 changes since V8
import { scan, map, filter, refCount, publishReplay } from 'rxjs/operators';
import { Thread } from "../thread/thread.model";
import { User } from "../user/user.model";

// Functions
const initalMessages: Message[] = [];

interface IMessagesOperation extends Function {
    (messages: Message[]): Message[];
}


@Injectable()
export class MessagesService {
//// 3 data management stream
// new message stream
newMessages: Subject<Message> = new Subject<Message>();

// array for the EXISTING messages Read Only
messages: Observable<Message[]>;  // messages: Message[]

// update messages
updates: Subject<any> = new Subject<any>(); // updates: any;

//// 2 action streams
// create
create: Subject<Message> = new Subject<Message>();
// isRead
markThreadAsRead: Subject<any> = new Subject<any>();

constructor() {
    this.messages = this.updates.pipe(
        scan( (messages: Message[], operation: IMessagesOperation) => {
            return operation(messages);
        }, initalMessages) // scan
        ) // pipe
        .pipe( publishReplay(1))
        .pipe( refCount());

    // Initial Messages concatinate the new message END
    this.create.pipe(
        map( function(message: Message): IMessagesOperation {
            return (messages: Message[]) => {
                return messages.concat(message);
            }; // return messages
        } // function
        ) // map
    ) // pipe
    .subscribe(this.updates);

    // new messages.create.updates.messages
    this.newMessages.subscribe(this.create);

    // markThreadAsRead
    this.markThreadAsRead
    .pipe(map( (thread: Thread) => {
      return (messages: Message[]) => {
        return messages.map( (message: Message) => {
          if (message.thread.id === thread.id) {
            message.isRead = true;
          }
          return message;
        });
      };
    })) // pipe
    .subscribe(this.updates);

} // constructor

// call to the actions stream
addMessage(message: Message): void  {

    this.newMessages.next(message);

} // addMessage

messagesForThreadUser(thread: Thread, user: User): Observable<Message> {
    return this.newMessages
      .pipe(filter((message: Message) => {
               // belongs to this thread
        return (message.thread.id === thread.id) &&
               // and isn't authored by this user
               (message.author.id !== user.id);
      })
      )//pipe
      ;
  } // messages for Thread

} // MessagesService

export const messagesServiceInjectables: Array<any> = [
    MessagesService
];