import { Injectable } from '@angular/core';
import * as _ from 'lodash';
import { BehaviorSubject, Observable, Subject } from 'rxjs';
import { Message } from '../message/message.model';
import { MessagesService } from '../message/messages.service';
import { Thread } from './thread.model';
import { map, combineLatest } from 'rxjs/operators'

@Injectable()
export class ThreadsService {
// threads observable
threads: Observable<{ [key:string] : Thread }>;

// sort the data stream objects
orderedThreads: Observable<Thread[]>;

// current thread
currentThread: Subject<Thread> = new BehaviorSubject<Thread>(new Thread());

// currentThread => messages
currentThreadMessages: Observable<Message[]>;

constructor(public messagesService: MessagesService) {

    this.threads = messagesService.messages
      .pipe(map( (messages: Message[]) => {
        const threads: {[key: string]: Thread} = {};
        // Store the message's thread in our accumulator `threads`
        messages.map((message: Message) => {
          threads[message.thread.id] = threads[message.thread.id] ||
            message.thread;

          // Cache the most recent message for each thread
          const messagesThread: Thread = threads[message.thread.id];
          if (!messagesThread.lastMessage ||
              messagesThread.lastMessage.sentAt < message.sentAt) {
            messagesThread.lastMessage = message;
          }
        });
        return threads;
      }) // map
      )  // pipe
      ;

    this.orderedThreads = this.threads
      .pipe(map((threadGroups: { [key: string]: Thread }) => {
        const threads: Thread[] = _.values(threadGroups);
        return _.sortBy(threads, (t: Thread) => t.lastMessage.sentAt).reverse();
      }) //map
      ) //pipe
      ;

    this.currentThreadMessages = this.currentThread
      .pipe(combineLatest(messagesService.messages,
                     (currentThread: Thread, messages: Message[]) => {
        if (currentThread && messages.length > 0) {
          return _.chain(messages)
            .filter((message: Message) =>
                    (message.thread.id === currentThread.id))
            .map((message: Message) => {
              message.isRead = true;
              return message; })
            .value();
        } else {
          return [];
        }
      }) // combineLatest
      ) // pipe
      ;

    this.currentThread.subscribe(this.messagesService.markThreadAsRead);
  }

    // function View => CC => DS
    // data stream => operations
    setCurrentThread(newThread: Thread) {
        this.currentThread.next(newThread);
    }


}